import bamnostic as bs
import argparse
import os.path
import json
import csv

from WholeGenomeAnalysis.SNP.model.AutoTypingResult import AutoTypingResult
from WholeGenomeAnalysis.SNP.model.Sample import Sample
from WholeGenomeAnalysis.SNP.model.Amplicon import Amplicon
from WholeGenomeAnalysis.SNP.process import CallSnip as callsnip, reader
from WholeGenomeAnalysis.SNP.process import QualityCalculation as qualityCalculation

parser = argparse.ArgumentParser(description='Input for Snip program')

parser.add_argument("-s", "--snpJsonpath", help='input snp json file path', required=True, dest="snpJsonpath")
parser.add_argument("-j", "--jsonpath", help='input json file path (output json from the sample processor program', required=True, dest="jsonPath")
parser.add_argument("-b", "--bampath", help='input bam file path', required=True, dest="bamPath")
parser.add_argument("-o", "--outfolder",help='output folder location', required=True, dest="outfolder")

args = parser.parse_args()

if (os.path.exists(args.snpJsonpath)):
    print("File exist : " + args.snpJsonpath)
else:
    print("Missing json file : " + args.snpJsonpath)
    exit()

if (os.path.exists(args.bamPath)):
    print("File exist : " + args.bamPath)
else:
    print("Missing bam file : " + args.bamPath)
    exit()

if (os.path.exists(args.jsonPath)):
    print("File exist : " + args.jsonPath)
else:
    print("Missing bam json : " + args.jsonPath)
    exit()

if (os.path.exists(args.outfolder)):
    print("Folder exist : " + args.outfolder)
else:
    print("Created path : " + args.outfolder)
    os.mkdir(args.outfolder)


snpJsonpath = args.snpJsonpath
bamPath = args.bamPath
jsonpath = args.jsonPath

snpData = reader.JsonReader(snpJsonpath)
qualityData = reader.JsonReader(jsonpath)
bam = bs.AlignmentFile(bamPath, 'rb')

# copiedbam = copy.copy(bam)

# for snp in data['SNP']:
#     print('name : ' + snp['name'])

snpVersion = snpData['SNPVERSION']
targetSnpVersion = snpData['VERSION']
experimentID = qualityData['ExperimentID']
experimentName = qualityData['ExperimentName']
plateName = qualityData['PlateName']
sampleId = qualityData['SampleID']


# Create output path

outputpath = args.outfolder+'/'+sampleId
#print("output path : " +outputpath)
#outputfile = writer.csvWriter(outputpath)


# sorted_bam = sorted(bam.head(), key=operator.itemgetter(1), reverse=True)

# for reading in enumerate(bamtmp):
#     print(reading)
# Result = {}
# AmpliconList = {}

# for sequence in copiedbam:
#     if (hasattr(sequence, 'reference_name')):
#         if sequence.query_name.startswith('SNP'):
#             AmpliconName = sequence.query_name.split('#')[2]
#             AmpliconList[AmpliconName] = Amplicon(AmpliconName)

autoResult = AutoTypingResult(snpVersion, targetSnpVersion)
smpl = Sample(experimentID, experimentName, plateName, sampleId)

with open(outputpath, mode='w', newline='') as csv_file:
    fieldnames = ['experimentName', 'sampleId', 'seqName', 'rawReads', 'snpName', 'chrom', 'chromStart', 'chromEnd',
                  'refNCBI', 'refUCSC', 'targeted', 'observed', 'minBaseOcurrances', 'minBasePercentages',
                  'minBaseQualityPercentage']
    writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
    writer.writeheader()

    for sequence in bam:
        if (hasattr(sequence, 'reference_name')):
            if sequence.query_name.startswith('SNP'):
                start = sequence.pos
                end = sequence.pos + len(sequence.seq)
                isTargeted = False
                # print("1. reference_name: " + sequence.reference_name + " | 1. sequence.query_name: " + sequence.query_name)
                # print("1. sequence.read_name: " + sequence.read_name)
                # dt = {}
                print("Name : " + sequence.query_name)
                AmpliconName = sequence.query_name.split('#')[2]
                amp = Amplicon(AmpliconName)

                for snip in snpData['SNP']:
                    if (snip['chrom'] == sequence.reference_name):

                        # print("2. reference_name: " + sequence.reference_name)v
                        # print("2. sequence.query_name: " + sequence.query_name)

                        chromStart = snip['chromStart']
                        chromEnd = snip['chromEnd']

                        if (start < chromStart < end and start < chromEnd < end):
                            # print("1. reference_name: " + sequence.reference_name + " | 1. sequence.query_name: " + sequence.query_name)

                            # AmpliconName = sequence.query_name.split('#')[2]
                            rawReads = sequence.query_name.split('#')[4]
                            targetPositionStart = chromStart - start
                            targetPositionEnd = chromEnd - start
                            print("targetPositionStart : " + str(targetPositionStart) + " | targetPositionEnd: " + str(targetPositionEnd))
                            calStart, calend = callsnip.CigarCalculation(targetPositionStart, targetPositionEnd, sequence.cigar)
                            print("start : " + str(calStart) + " | end: " + str(calend))
                            rawbases = sequence.seq[calStart:calend]

                            if(calStart==calend and rawbases==''):
                                rawbases='-'

                            minBaseOcurrances, minBasePercentages, minBaseQualityPercentage = qualityCalculation.getQuality(qualityData, rawbases, calStart, calend, sequence.query_name)
                            #rawbases = sequence.seq[targetPositionStart:targetPositionEnd]

                            #snps = Snp(1, sequence.query_name, rawReads, snip['name'], snip['chrom'], chromStart, chromEnd,
                            #           snip['refNCBI'], snip['refUCSC'], snip['observed'], rawbases, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence)

                            # dt.update(vars(snps))
                            # dt = json.dumps(dt)

                            #amp.add(snps)
                            writer.writerow({'experimentName': experimentName, 'sampleId': sampleId, 'seqName': sequence.query_name, 'rawReads': rawReads, 'snpName': snip['name'], 'chrom': snip['chrom'], 'chromStart': str(chromStart), 'chromEnd': str(chromEnd), 'refNCBI': snip['refNCBI'], 'refUCSC': snip['refUCSC'], 'targeted': snip['observed'], 'observed': rawbases, 'minBaseOcurrances': str(minBaseOcurrances), 'minBasePercentages': str(minBasePercentages), 'minBaseQualityPercentage': str(minBaseQualityPercentage)})

                            print(experimentName + ',' + sampleId + ',' + sequence.query_name + ',' + rawReads + ',' + snip[
                                'name'] + ',' + snip['chrom'] + ',' + str(chromStart) + ',' + str(chromEnd) + ',' + snip[
                                      'refNCBI'] + ',' + snip['refUCSC'] + ',' + snip['observed'] + ',' + rawbases  + ',' + str(minBaseOcurrances) + ',' + str(minBasePercentages) + ',' + str(minBaseQualityPercentage))
                            isTargeted = True
                            break

                # AmpliconName = sequence.query_name.split('#')[2]
                # amp = Amplicon(AmpliconName)

                # if AmpliconName in AmpliconList:
                #     amp = AmpliconList[AmpliconName] # Get Amplicon object by AmpliconName
                #     amp.add(dt)

                #if isTargeted == True:
                #    smpl.add(amp)
                #else:
                #    print("None Target List: " + sequence.query_name)

autoResult.add(smpl)

autoResult.autoTyping_dic = {}
autoResult.autoTyping_dic["snpVersion"] = autoResult.snpVersion
autoResult.autoTyping_dic["targetSnpVersion"] = autoResult.targetSnpVersion
autoResult.autoTyping_dic["sample"] = autoResult.sample_list

Convert_Json = {
    "autoTypingResult": autoResult.autoTyping_dic
}

jsonFormat = json.dumps(Convert_Json, indent=4)
print(jsonFormat)




import copy


class Sample:

    def __init__(self, experimentID, experimentName, plateName, sampleId):
        self.experimentID = experimentID
        self.experimentName = experimentName
        self.plateName = plateName
        self.sampleId = sampleId
        self.Amplicon = []
        self.amplicon_dic = {}
        self.amplicon_list = []

    def get(self):
        return self.Amplicon

    def add(self, Amplicon):
        self.Amplicon.append(Amplicon)
        self.amplicon_dic = copy.deepcopy(self.amplicon_dic)
        self.amplicon_dic["ampliconName"] = Amplicon.ampliconName
        self.amplicon_dic["snp"] = Amplicon.snp_list
        self.amplicon_list.append(self.amplicon_dic)

    def reprJSON(self):
        return dict(experimentID=self.experimentID, experimentName=self.experimentName, plateName=self.plateName, sampleId=self.sampleId, Amplicon=self.Amplicon)

    # def make_sample(experimentID, experimentName, plateName, sampleId):
#     sample = Sample(experimentID, experimentName, plateName, sampleId)
#     return sample



def getSnipPosition(bam,jsondata):
    for sequence in bam:
        if (hasattr(sequence, 'reference_name')):
            print(sequence.reference_name)
            for snp in jsondata['SNP']:
                if(snp['chrom'] == sequence.reference_name):
                    print


    for sequence in bam:
        if (hasattr(sequence, 'reference_name')):
            start=sequence.pos
            end=sequence.pos+len(sequence.seq)
            for snp in data['SNP']:
                if(snp['chrom'] == sequence.reference_name):
                    chromStart=snp['chromStart']
                    chromEnd=snp['chromEnd']
                    if(start < chromStart <  end and start < chromEnd < end):
                        AmpliconName=sequence.query_name.split('#')[2]
                        rawReads = sequence.query_name.split('#')[4]
                        targetPositionStart=chromStart-start
                        targetPositionEnd=chromEnd-start

                        rawbases=sequence.seq[targetPositionStart:targetPositionEnd]
                        print(BlotName+','+SampleName+','+AmpliconName+','+rawReads+','+snp['name']+','+snp['chrom']+','+str(chromStart)+','+str(chromEnd)+','+snp['refNCBI']+','+snp['refUCSC']+','+snp['observed']+','+rawbases)

def CigarCalculation(targetPositionStart, targetPositionEnd, cigar):
    calPositionStart = targetPositionStart
    returnStart = targetPositionStart
    returnEnd = targetPositionEnd

    # Cigar value 0 is mutation, 1 is Insertion , 2 is Deletion.
    for arrCigar in cigar:
        mutationType=arrCigar[0]
        value=arrCigar[1]

        if(value < calPositionStart):
            if(mutationType==0):
                calPositionStart=calPositionStart-value
            elif(mutationType==1):
                returnStart = targetPositionStart + value
                returnEnd = targetPositionEnd + value
                calPositionStart = targetPositionStart + value
            elif(mutationType==2):
                returnStart=targetPositionStart-value
                returnEnd=targetPositionEnd-value
                calPositionStart=targetPositionStart-value
        else:
            return returnStart,returnEnd

    return returnStart, returnEnd



import argparse
import os.path
from WholeGenomeAnalysis import reader
from WholeGenomeAnalysis import QualityCalculation as qualityCalculation

parser = argparse.ArgumentParser(description='Input for Snip program')

parser.add_argument("-j", "--jsonpath", help='input json file path (output json from the sample processor program', required=True, dest="jsonPath")

args = parser.parse_args()

if (os.path.exists(args.jsonPath)):
    print("File exist : " + args.jsonPath)
else:
    print("Missing bam json : " + args.jsonPath)
    exit()

jsonpath = args.jsonPath

qualityData = reader.JsonReader(jsonpath)
# copiedbam = copy.copy(bam)

# for snp in data['SNP']:
#     print('name : ' + snp['name'])

snpVersion = 151
targetSnpVersion = 1

rawbase='AG'
start=236
end=238

arrBases = list(rawbase)
#print(arrBases)
minBaseOcurrances ,minBasePercentages, minBaseQualityPercentage = qualityCalculation.getQuality(qualityData,rawbase,start,end)

print (minBaseOcurrances ,minBasePercentages, minBaseQualityPercentage )

#for amplicon in qualityData['Amplicon']:
#    for allele in amplicon['Allele']:
#        if (allele['SequenceName'] == "SNP#41#AMPL788534#1#596"):
#            for i in range(start, end):
#                print(i)
#                for k,v in allele['BaseOcurrances'].items():
#                    if(k==rawbase):
#                        x = v.split(",")
#                        print(x[i])
#                        break
#                for k,v in allele['BasePercentages'].items():
#                    if(k==rawbase):
#                        x = v.split(",")
#                        print(x[i])
#                        break
#                for k,v in allele['BaseQualityPercentage'].items():
#                    if(k==rawbase):
#                        x = v.split(",")
#                        print(x[i])
#                        break
            #print(allele['BaseOcurrances'])
            #print(allele['BaseQuality'])
            #print(allele['BasePercentages'])
            #print(allele['BaseQualityPercentage'])
            #print(allele['AverageQuality'])